@@ -1,3 +1,4 @@
+from nonebot_plugin_apscheduler import scheduler
 import nonebot
 from nonebot.plugin import on_command
 from nonebot.adapters.onebot.v11 import PrivateMessageEvent, GroupMessageEvent, Message, MessageEvent
@@ -10,15 +11,16 @@ from builtins import ConnectionError
 from pathlib import Path
 import asyncio
 from datetime import datetime
+from dateutil.parser import parse
 import os
-from .model import check, cron_check, get_sport_record, get_timetable, get_whole_day_course, get_next_course, get_question, des_encrypt, des_descrypt
+from .model import check, cron_check, get_sport_record, get_timetable, get_whole_day_course, get_next_course, get_question, get_teaching_buildings, get_classroom, get_idle_classroom, des_encrypt, des_descrypt, analyse_best_idle_room
 from .config import Config
+import json
 
 # 启动定时器---------------------------------------------------------------
 
 require("nonebot_plugin_apscheduler")
 
-from nonebot_plugin_apscheduler import scheduler
 
 # 配置初始项---------------------------------------------------------------
 
@@ -27,7 +29,8 @@ MODLE = {
     "体育打卡": "Sports",
     "课表提醒": "Ehall",
     "选课操作": "XK",
-    "马原测试": "MY"
+    "马原测试": "MY",
+    "空闲教室查询":"Ehall"
 }
 
 MODEL_NEED = {
@@ -40,9 +43,11 @@ MODEL_NEED = {
 MODEL_RUN_TIME = {
     "晨午晚检": "被动：每天7/13/20点自动打卡，定位在南校区\n主动（晨午晚检查看/查看晨午晚检）：返回本阶段是否已打卡",
     "体育打卡": "被动：每10分钟检测一次，如果您有已上报的正在打卡的记录将会提醒您\n主动（体育打卡查看/查看体育打卡）：返回当前打卡次数信息",
-    "课表提醒": "被动：每天早上7点私聊提醒一次今天课表上的课及其位置，每节课前30分钟提醒下节有课\n主动（）：返回当天课表以及此时有没有课，最近的课是在什么时候",
+    "课表提醒": "被动：每天早上7点私聊提醒一次今天课表上的课及其位置，每节课前30分钟提醒下节有课\n主动（空闲教室）：返回当天课表以及此时有没有课，最近的课是在什么时候",
     "选课操作": "暂未编写，但在计划内",
-    "马原测试": "被动：无\n主动：返回一道单选或者多选题,可以通过参数决定，无参数默认随机"}
+    "马原测试": "被动：无\n主动：返回一道单选或者多选题,可以通过参数决定，无参数默认随机",
+    "空闲教室查询": "主动:返回查找日期的空闲教室，会经过优化"
+}
 
 TIME_SCHED = [
     ("8:30", "10:05"),
@@ -74,6 +79,9 @@ XDU_SUPPORT_PATH = os.path.join(xdu_support_path, "user_data")
 if not os.path.exists(XDU_SUPPORT_PATH):
     os.makedirs(XDU_SUPPORT_PATH)
 
+if not os.path.exists(os.path.join(XDU_SUPPORT_PATH, "idle_classroom_query")):
+    os.makedirs(os.path.join(XDU_SUPPORT_PATH, "idle_classroom_query"))
+
 for name in MODLE.values():
     if not os.path.exists(os.path.join(XDU_SUPPORT_PATH, f"{name}.txt")):
         f = open(os.path.join(XDU_SUPPORT_PATH, f"{name}.txt"), 'w')
@@ -109,6 +117,7 @@ sport_punch = on_command("体育打卡查看", priority=6, block=False, aliases=
 timetable = on_command("课表查询", priority=6, block=False, aliases={"课表查看"})
 update_timetable = on_command("更新课表", priority=6, block=False)
 mayuan = on_command("马原", priority=6, block=True)
+idle_classroom_query = on_command("空闲教室", priority=5, block=True)
 
 # 功能订阅-----------------------------------------------------------------
 
@@ -127,6 +136,7 @@ async def handle(event: PrivateMessageEvent, state: T_State, args: Message = Com
         res += "==========================\n"
         for model, model_run_time in MODEL_RUN_TIME.items():
             res += f"{model}功能的用法为{model_run_time}\n\n"
+        res += f"更多信息请查看https://longchengguxiao.github.io/plugindoc/#/nonebot_plugin_xdu_support/README"
         await add_sub.send(res)
         await asyncio.sleep(0.5)
 
@@ -378,6 +388,7 @@ async def _(event: MessageEvent):
                 f'{username}-remake.json')):
             await timetable.send("未找到本地课表，正在进行在线爬取并储存，请稍等", at_sender=True)
             ses = EhallSession(username, password)
+            ses.use_app(4770397878132218)
             get_timetable(ses, username, XDU_SUPPORT_PATH)
             await timetable.send("课表更新完成，启动自动提醒，稍后返回数据", at_sender=True)
         message += get_whole_day_course(username, TIME_SCHED, XDU_SUPPORT_PATH)
@@ -558,6 +569,131 @@ async def _(event: MessageEvent, state: T_State, user_ans: str = ArgStr("user_an
     _ = write_data(path, users)
     await mayuan.finish(res_after_ans)
 
+# 空闲教室查询------------------------------------------------------------------------
+
+
+@idle_classroom_query.handle()
+async def _(event: MessageEvent, state: T_State, args: Message = CommandArg()):
+    flag, users = read_data(Path(XDU_SUPPORT_PATH, 'Ehall.txt'))
+    users_id = [x[0] for x in users]
+    user_id = str(event.user_id)
+    message = ""
+    app_id = 4768402106681759
+    if user_id in users_id:
+        username = users[users_id.index(user_id)][1]
+        password = des_descrypt(
+            users[users_id.index(user_id)][2], DES_KEY).decode()
+        ses = EhallSession(username, password)
+        ses.use_app(app_id)
+        if not os.path.exists(
+            os.path.join(
+                XDU_SUPPORT_PATH,
+                "teaching_buildings.txt")):
+            teaching_buildings = get_teaching_buildings(ses)
+            with open(os.path.join(XDU_SUPPORT_PATH, "idle_classroom_query", "teaching_buildings.txt"), 'w', encoding="utf-8") as f:
+                f.write(str(teaching_buildings))
+        else:
+            with open(os.path.join(XDU_SUPPORT_PATH, "idle_classroom_query", "teaching_buildings.txt"), 'r', encoding="utf-8") as f:
+                teaching_buildings = eval(f.read())
+        buildings = [x[0] for x in teaching_buildings]
+        msg = args.extract_plain_text().strip().split(" ")
+        if len(msg) == 1 and msg[0] in buildings:
+            state["build"] = msg[0]
+        elif len(msg) == 2:
+            if msg[0] in buildings:
+                state["build"] = msg[0]
+            try:
+                time_select = parse(msg[1], fuzzy=False)
+                state["time_select"] = str(time_select).split(' ')[0]
+            except ValueError:
+                pass
+        else:
+            message += "南校区的教学楼有:\n"
+            for build in teaching_buildings:
+                message += f"{build[0]},"
+            await idle_classroom_query.send(message)
+        state["ses"] = ses
+        state["buildings"] = teaching_buildings
+    else:
+        await timetable.finish("请先订阅空闲教室功能，再进行查询")
+
+
+@idle_classroom_query.got("build", prompt="请输入您要查找的教学楼名称")
+async def _(state: T_State, build: str = ArgStr("build")):
+    if build in ["取消", "算了"]:
+        await idle_classroom_query.finish("已取消本次操作")
+    ses = state["ses"]
+    teaching_buildings = state["buildings"]
+    buildings = [x[0] for x in teaching_buildings]
+    if build in buildings:
+        if not os.path.exists(
+            os.path.join(
+                XDU_SUPPORT_PATH,
+                f"{build}-classroom.txt")):
+            rooms = get_classroom(
+                ses, teaching_buildings[buildings.index(build)][1])
+            with open(os.path.join(XDU_SUPPORT_PATH, "idle_classroom_query", f"{build}-classroom.txt"), 'w', encoding='utf-8') as f:
+                f.write(str(rooms))
+        else:
+            with open(os.path.join(XDU_SUPPORT_PATH, "idle_classroom_query", f"{build}-classroom.txt"), 'r', encoding='utf-8') as f:
+                rooms = eval(f.read())
+
+        state["rooms"] = rooms
+        state["ses"] = ses
+        state["build"] = build
+    else:
+        await idle_classroom_query.finish("您输入的教学楼名称有误，请检查输入")
+
+
+@idle_classroom_query.got("time_select", prompt="请输入查询日期,格式为年-月-日")
+async def _(event:MessageEvent, state: T_State, time_selector: str = ArgStr("time_select")):
+    message = ""
+    if time_selector in ["取消", "算了"]:
+        await idle_classroom_query.finish("已取消本次操作")
+    try:
+        time_ = parse(time_selector)
+        time_ = str(time_).split(" ")[0]
+        ses = state["ses"]
+        rooms = state["rooms"]
+        build = state["build"]
+        teaching_buildings = state["buildings"]
+        buildings = [x[0] for x in teaching_buildings]
+        if not os.path.exists(
+            os.path.join(
+                XDU_SUPPORT_PATH,
+                "idle_classroom_query",
+                f"{time_}_{build}_idle_rooms.txt")):
+            result = await get_idle_classroom(ses, rooms, time_)
+            with open(os.path.join(XDU_SUPPORT_PATH, "idle_classroom_query", f'{time_}_{build}_idle_rooms.txt'), 'w', encoding='utf-8') as f:
+                f.write(str(result))
+        else:
+            with open(os.path.join(XDU_SUPPORT_PATH, "idle_classroom_query", f'{time_}_{build}_idle_rooms.txt'), 'r', encoding='utf-8') as f:
+                result = eval(f.read())
+        flag, users = read_data(Path(XDU_SUPPORT_PATH, 'Ehall.txt'))
+        users_id = [x[0] for x in users]
+        user_id = str(event.user_id)
+        message = ""
+        if user_id in users_id:
+            username = users[users_id.index(user_id)][1]
+            password = des_descrypt(
+                users[users_id.index(user_id)][2], DES_KEY).decode()
+            if not os.path.exists(
+                    os.path.join(
+                        XDU_SUPPORT_PATH,
+                        f'{username}-remake.json')):
+                await timetable.send("未找到本地课表，正在进行在线爬取并储存，请稍等", at_sender=True)
+                ses = EhallSession(username, password)
+                ses.use_app(4770397878132218)
+                get_timetable(ses, username, XDU_SUPPORT_PATH)
+                await timetable.send("课表更新完成，启动自动分析，稍后返回数据", at_sender=True)
+            with open(os.path.join(XDU_SUPPORT_PATH, f"{username}-remake.json"), "r", encoding="utf-8") as f:
+                courses = json.loads(f.read())
+        message = analyse_best_idle_room(result, courses, time_, teaching_buildings[buildings.index(build)][1])
+        await idle_classroom_query.finish(message)
+    except ValueError:
+        await idle_classroom_query.finish("输入日期有误，请检查输入")
+
+
 # 文档操作----------------------------------------------------------------------------
 
 
