@@ -17,6 +17,7 @@ import binascii
 from requests.cookies import RequestsCookieJar
 import httpx
 from collections import Counter
+from dateutil.parser import parse
 
 # 晨午晚检------------------------------------------------------------------------
 
@@ -175,13 +176,13 @@ def get_timetable(ses: EhallSession, username: str,
             if courseList[week_cnt][day_cnt]:
                 date = termStartDay + \
                     timedelta(days=week_cnt * 7 + day_cnt)  # 从第一 周的第一天起
-                remake[f"{date.year}-{date.month}-{date.day}"] = {}
+                remake[str(parse(f"{date.year}-{date.month}-{date.day}")).split(" ")[0]] = {}
                 for course in courseList[week_cnt][day_cnt]:
                     if course['sectionSpan'][0] > 10:
                         continue
                     elif course['location'] is None:
                         course['location'] = '待定'
-                    remake[f"{date.year}-{date.month}-{date.day}"][int(course['sectionSpan'][1] / 2 - 1)] = {
+                    remake[str(parse(f"{date.year}-{date.month}-{date.day}")).split(" ")[0]][int(course['sectionSpan'][1] / 2 - 1)] = {
                         'name': course['name'],
                         'location': course['location'],
                     }
@@ -196,9 +197,9 @@ def get_next_course(username: str, basic_path: Union[Path, str]) -> str:
     with open(os.path.join(basic_path, f"{username}-remake.json"), "r", encoding="utf-8") as f:
         courses = json.loads(f.read())
     today = datetime.now()
-    if courses.get(f"{today.year}-{today.month}-{today.day}", None):
+    if courses.get(str(parse(f"{today.year}-{today.month}-{today.day}")).split(" ")[0], None):
         today_course = courses.get(
-            f"{today.year}-{today.month}-{today.day}", None)
+            str(parse(f"{today.year}-{today.month}-{today.day}")).split(" ")[0], None)
         if today.hour == 8:
             if today_course.get(0, None):
                 course = today_course.get(0)
@@ -233,10 +234,10 @@ def get_whole_day_course(username: str, time_sche: List,
     d = today.day
     if _time == 1:
         d += 1
-    if courses.get(f"{y}-{m}-{d}", None):
+    if courses.get(str(parse(f"{y}-{m}-{d}")).split(" ")[0], None):
 
         today_course: Dict = courses.get(
-            f"{y}-{m}-{d}", None)
+            str(parse(f"{y}-{m}-{d}")).split(" ")[0], None)
         if _time == 0:
             message += f"今天一共有{len(list(today_course.keys()))}结课需要上\n"
         else:
@@ -401,7 +402,7 @@ def analyse_best_idle_room(idle_room:Dict[str, List], timetable:Dict[str, Dict[i
             # 有同一栋楼
             if course_buildings[i] == building:
                 flag = 1
-                course_time = list(today_course.keys())[i]
+                course_time = int(list(today_course.keys())[i])
                 course_floor = course_rooms[i][0]
                 if course_time != 4:
                     for room in idle_room[time_sche[course_time+1]]:
@@ -413,6 +414,8 @@ def analyse_best_idle_room(idle_room:Dict[str, List], timetable:Dict[str, Dict[i
                 collection_rooms = Counter(result)
                 best_ans = collection_rooms.most_common(1)
                 message += f"结合您{time_}的课表推荐您去{building}的{best_ans[0][0]}教室自习，离您的教室较近且空的时间较多\n"
+        if flag == 0:
+            message += f"结合您{time_}的课表，您在当天有课，但不在{building},即将为您推荐{building}空闲时间最多的教室。但可能更换自习地点会有更好的选择哦\n"
     else:
         message += f"鉴于您{time_}的课表没有课，已为您推荐当天{building}空闲时间最多的教室\n"
     ans = []
